// Reconcile DEP-5 debian/copyright to licensecheck
//
// Copyright : 2023-2025 P Blackman
// License   : BSD-2-clause
//

unit compare;
{$mode delphi}

interface

procedure CompareLicenses;


implementation uses rstrings, options, filedata;


procedure CompareLicenses;
var GotOne : Boolean;
begin
    GotOne := CheckFiles;

    if GotOne then
    begin
        Writeln;

        if Option_Short then
        begin
            Writeln ('Short ' + rsSiu); // Not all differences shown
            Writeln;
        end;

        Halt (3);
    end
    else
    if not Option_Format then
    begin
        if Option_Long then
            Writeln;
        Writeln (rsNdf); // No differences found
    end;
end;

end.
