Copyright (c) 2026 by Peter. All rights reserved.
This program is free software;
you can redistribute it and/or modify it
under the same terms as Perl itself.
This software comes with <NO WARRANTY> of any kind.
